/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

/**
 * DirectActivationResponse
 */
class DirectActivationResponse {
    
    private static let FIELD_ENDPOINT_ID: String       = "endpointId"
    private static let FIELD_ACTIVATION_TIME: String   = "activationTime"
    private static let FIELD_ENDPOINT_STATE: String    = "endpointState"
    private static let FIELD_CERTIFICATE: String       = "certificate"
    
    private var endpointId: String?
    private var activationTime: Date?
    private var endpointState: String?
    
    private var certificate: Foundation.Data?
    
    func getCertificate() -> Foundation.Data {
        return certificate!
    }
    
    func setCertificate(certificate: Foundation.Data) {
        self.certificate = certificate
    }
    
    init() {}
    
    func getActivationTime() -> Date {
        return activationTime!
    }
    
    func setActivationTime(activationTime: Date) {
        self.activationTime = activationTime
    }
    
    func getEndpointState() -> String {
        return endpointState!
    }
    
    func setEndpointState(endpointState: String) {
        self.endpointState = endpointState
    }
    
    func getEndpointId() -> String {
        return endpointId!
    }
    
    func setEndpointId(endpointId: String) {
        self.endpointId = endpointId
    }
    
    func toString() -> String {
        var result: String = "ActivationResponse{"
        result = result + "endpointId='" + endpointId! + "'"
        result = result + ", activationTime=" + (activationTime?.description)!
        result = result + ", endpointState='" + endpointState! + "'"
        return result
    }
    
    private static func parseJsonStr(str: String) throws -> [String : AnyObject]? {
        if let data = str.data(using: .utf8) {
            return try JSONSerialization.jsonObject(with: data, options: []) as? [String : AnyObject]
        }
        return nil
    }
    
    static func fromJson(data: Foundation.Data?) throws -> DirectActivationResponse? {
        if data == nil {
            return nil
        }
        let jsonStr: String = String(data: data!, encoding: .utf8)!
        let jsonDict: [String: AnyObject]? = try parseJsonStr(str: jsonStr)
        if jsonDict == nil {
            print("DirectActivationResponse is empty")
            return nil
        }
        
        let response: DirectActivationResponse = DirectActivationResponse()
        response.endpointId = jsonDict![FIELD_ENDPOINT_ID] as? String
    
        // TODO: implement a Date converter
        response.endpointState = jsonDict![FIELD_ENDPOINT_STATE] as? String
    
        // TODO: we're manually replacing EOLs here, need to figure out a better solution
        var fieldCertificate: String? = (jsonDict![FIELD_CERTIFICATE] as? String)
        if (fieldCertificate != nil) {
            var toArray = fieldCertificate!.components(separatedBy: "\r")
            fieldCertificate = toArray.joined(separator: "")
            toArray = fieldCertificate!.components(separatedBy: "\n")
            fieldCertificate = toArray.joined(separator: "")
            let certificateData: Foundation.Data = Foundation.Data(base64Encoded: fieldCertificate!,
                    options: Foundation.Data.Base64DecodingOptions(rawValue: 0))!
            response.certificate = certificateData
        } else {
            response.certificate = Foundation.Data(bytes: UnsafePointer<UInt8>([]), count: 0)
        }
    
        return response;
    }
    
    func toJsonString() -> String {
        var resultStr: String = "{"
        resultStr = resultStr + "\"" + "FIELD_ENDPOINT_ID" + "\":\"" + endpointId! + "\","
        resultStr = resultStr + "\"" + "FIELD_ENDPOINT_STATE" + "\":\"" + endpointState!  + "\","
        resultStr = resultStr + "\"" + "FIELD_ACTIVATION_TIME" + "\":" + activationTime!.description(with: .autoupdatingCurrent) + "}"
        return resultStr
    }
}
