/*
 * Copyright (c) 2017, Oracle and/or its affiliates.  All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and
 * the Universal Permissive License (UPL).  See the LICENSE file in the root
 * directory for license terms.  You may choose either license, or both.
 */

// com.oracle.iot.client.impl.device

import Foundation

/**
 * Number of bytes transferred between progress updates
 */
fileprivate let PROGRESS_UPDATE_INTERVAL:Int64 = 1024*1024

/**
 * `StorageObjectDelegate` extends `StorageObjectDelegateBase` to provide extra state and
 * data for working with `StorageConnectionImpl`.
 */
public class StorageObjectDelegate : StorageObjectDelegateBase {

    private /*volatile*/ var state:ProgressState!

    override init(storageConnection:StorageConnection,
            uri:String, name:String, contentType:String?,
            contentEncoding:String?,
            dateOfLastModification:Date?, length:Int64) {
        
        self.state = .unknown
        super.init(storageConnection:storageConnection, uri:uri, name:name, 
                contentType:contentType, contentEncoding:contentEncoding, 
                dateOfLastModification:dateOfLastModification, length:length)

    }

    func setState(state:ProgressState) {
        DispatchQueue.global(qos: .utility).sync(execute: {
            self.state = state
        })
    }

    func getState() -> ProgressState {
        return DispatchQueue.global(qos: .utility).sync(execute: {
            return self.state
        })
    }

    public override func setTransferredBytes(transferredBytes:Int64) {
        let diff:Int64 = transferredBytes - self.transferredBytes
        super.setTransferredBytes(transferredBytes:transferredBytes)
        if diff >= PROGRESS_UPDATE_INTERVAL {
            // TODO...
        }
    }
}
