/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

class HttpClientUserAuth : HttpClient {
    
    let CSRF_TOKEN_REQUEST_PATH = "/iot/webapi/v2/private/server"
    let CSRF_TOKEN_NAME = "X-CSRF-TOKEN"
    
    typealias HttpResponseRetriever = () throws -> HTTPURLResponse
    
    func getCsrfToken(serverScheme: String, serverHost: String, serverPort: String, httpResponse: @escaping (HTTPURLResponse?, ClientError?) -> Void) {
        let baseURL = serverScheme + "://" + serverHost + ":" + serverPort
        let url = URL(string: baseURL + CSRF_TOKEN_REQUEST_PATH)
        
        let session = URLSession.shared
        session.configuration.httpShouldSetCookies = true
        session.configuration.httpCookieAcceptPolicy = .onlyFromMainDocumentDomain
        session.configuration.httpCookieStorage?.cookieAcceptPolicy = .onlyFromMainDocumentDomain
        
        var request = URLRequest(url: url!)
        request.httpMethod = "GET"
        request.addValue("application/json", forHTTPHeaderField: "Accept")
        request.addValue("application/json", forHTTPHeaderField: "Content-Type")
        
        var responseObject: HTTPURLResponse!

        let task = session.dataTask(with: request, completionHandler: {
            (data, response, error) -> Void in
            
                responseObject = response as? HTTPURLResponse
            
                if error != nil {
                    httpResponse(nil, ClientError.failedAuthentication((error?.localizedDescription)!))
                    return
                } else {
                    if responseObject.allHeaderFields["X-CSRF-TOKEN"] == nil {
                        httpResponse(nil, ClientError.failedAuthentication(NSLocalizedString("User Authentication failed! No X-CSRF-TOKEN.", comment: "")))
                        return
                    }
                    else {
                        if responseObject.statusCode == 302 {
                            httpResponse(nil, ClientError.failedAuthentication(NSLocalizedString("User Authentication failed!", comment: "")))
                            return
                        } else {
                            httpResponse(responseObject, nil)
                            return
                        }
                    }
                }
            }) 
        
        task.resume()
    }
}
