/*
 * Copyright (c) 2017, Oracle and/or its affiliates.  All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and
 * the Universal Permissive License (UPL).  See the LICENSE file in the root
 * directory for license terms.  You may choose either license, or both.
 */

// com.oracle.iot.client.impl.enterprise
/**
 * StorageObjectImpl
 */
public class StorageObjectImpl : StorageObjectBase {

    // The name of an attribute or a field
    private var nameForSyncEvent:String?
    private var deviceForSyncEvent:VirtualDevice?

    override init(delegate:AdvancedStorageObject) {
        super.init(delegate:delegate)
    }
    // This method is called from clients that are sync'ing
    // storage objects
    public override func sync() throws {
        try self.sync(callback: { (error) in
        })
    }
    // This method is called from VirtualDeviceImpl and needs a callback in
    // case of errors.
    func sync(callback: @escaping (ClientError?)->()) throws {
        if getSyncStatus() == SyncStatus.notInSync {
            try super.sync()
            StorageObjectBase.getDispatcher().async(execute: {
                do {
                    try self.getDelegate().sync(callback: { (error) in
                        if error == nil {
                            // Order is important.
                            self.setSyncStatus(syncStatus:SyncStatus.inSync)
                            self.addSyncEvent(syncEvent:self.createSyncEvent())
                        } else {
                            print(error!)
                            // Order is important.
                            self.setSyncStatus(syncStatus:SyncStatus.syncFailed)
                            self.addSyncEvent(syncEvent:self.createSyncEvent())
                        }
                        callback(error)
                    })
                } catch {
                    // Order is important.
                    self.setSyncStatus(syncStatus:SyncStatus.syncFailed)
                    self.addSyncEvent(syncEvent:self.createSyncEvent())
                    callback(ClientError.network("\(error)"))
                }
            })
        } else {
            try super.sync()
        }
    }

    func setSyncEventInfo(virtualDevice:VirtualDevice, nameForSyncEvent:String) {
        self.deviceForSyncEvent = virtualDevice
        self.nameForSyncEvent = nameForSyncEvent
        
        //enterprise side creates event here
        addSyncEvent(syncEvent:createSyncEvent())
    }
    
    override func handleStateChange() {
    }

    override func createSyncEvent() -> SyncEvent {
        return SyncEventImpl(source:self, virtualDevice:deviceForSyncEvent,
                             name:nameForSyncEvent)
    }

}
