/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation


class VirtualDeviceBase {
    private var syncQueue:DispatchQueue = DispatchQueue.global(qos: .utility)
    
    private var endpointId: String?
    
    private var deviceModel: DeviceModel?
    
    private var onChange: ((ChangeEvent) -> ())?
    private var onError: ((ErrorEvent) -> ())?
    
    init(endpointId: String, deviceModel: DeviceModel) {
        self.endpointId = endpointId
        self.deviceModel = deviceModel
    }
    
    func getDeviceModel() -> DeviceModel {
        return self.deviceModel!
    }
    
    func getEndpointId() -> String {
        return self.endpointId!
    }

    func setOnChange(callback: ((ChangeEvent) -> ())?) {
        syncQueue.sync(execute: {
            self.onChange = callback
        })
    }
    
    func getOnChangeCallback() -> ((ChangeEvent) -> ())? {
        return syncQueue.sync(execute: {
            return self.onChange
        })
    }
    
    func setOnError(callback: @escaping (ErrorEvent) -> ()) {
        syncQueue.sync(execute: {
            self.onError = callback
        })
    }
    
    func getOnErrorCallback() -> ((ErrorEvent) -> ())? {
        return syncQueue.sync(execute: {
            return self.onError
        })
    }
}
