/*
 *  Copyright © 2016 Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

/**
 * The error definitions for the Device library.
 */
public enum ClientError : LocalizedError {
    /**
     * Identifies file operation errors.
     */
    case file(String)
    /**
     * Identifies errors due to invalid object state, such calling activate
     * on an already activated device or if an instance member does not have
     * a valid value.
     */
    case state(String)
    /**
     * Identifies errors due it invalid method arguments, such as passing
     * Int to a device set method when a String is expected.
     */
    case argument(String)
    /**
     * Identifies network operation errors or if an http response is expected
     * but not available from a successful request.
     */
    case network(String)
    /**
     * Identifies security or trust operation errors.
     */
    case security(String)
    /**
     * Identifies errors due to failed authentication.
     */
    case failedAuthentication(String)

    var localizedDescription: String {
        switch self {
            case .file(let str):
                return str
            case .state(let str):
                return str
            case .argument(let str):
                return str
            case .network(let str):
                return str
            case .security(let str):
                return str
            case .failedAuthentication(let str):
                return str
        }
    }

}

