/*
 *  Copyright © 2016 Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

/**
 * Detailed information on a device model. A device model is a specification
 * of the attributes, formats, and resources available on the device.
 *
 */
public protocol DeviceModel {
    
    /**
     * Get the URN of the device model
     *
     * - returns: the URN of the model
     */
    func getURN() -> String
    
    /**
     * Get the name of the device model
     *
     * - returns: the device model name
     */
    func getName() -> String
    
    /**
     * Get the free form description of the device model
     *
     * - returns: the description of the model
     */
    func getDescription() -> String
}
