/*
 * Copyright (c) 2017, Oracle and/or its affiliates.  All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and
 * the Universal Permissive License (UPL).  See the LICENSE file in the root
 * directory for license terms.  You may choose either license, or both.
 */

// com.oracle.iot.client.impl

import Foundation

/**
 * StorageObjectDelegate extends AdvancedStorageObject to provide extra state and
 * data for working with StorageConnectionImpl.
 */
public class StorageObjectDelegateBase : AdvancedStorageObject {

    private var storageConnection:StorageConnection
    var transferredBytes:Int64

    init(storageConnection:StorageConnection, uri:String, name:String,
                                 contentType:String?, contentEncoding:String?,
                                 dateOfLastModification:Date?, length:Int64) {
        self.storageConnection = storageConnection
        self.transferredBytes = 0
        super.init(uri:uri, name:name, contentType:contentType,
                   encoding:contentEncoding,
                   date:dateOfLastModification,
                   length:length)
    }

    override func setMetadata(date:Date, length:Int64) {
        super.setMetadata(date:date, length:length)
    }

    
    public override func sync(callback: @escaping (ClientError?) -> ()) throws {
        try storageConnection.sync(storageObject: self, callback: callback)
    }

    public func isCanceled() -> Bool {
        // TODO:
        return false
    }

    public func setTransferredBytes(transferredBytes:Int64) {
        self.transferredBytes = transferredBytes
    }
}
