/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation
import SwiftyJSON

class SubjectPublicKeyInfo {
    static let FIELD_ALGORITHM: String = "algorithm"
    static let FIELD_PUBLIC_KEY: String = "publicKey"
    static let FIELD_FORMAT: String = "format"
    static let FIELD_SECRET_HASH_ALGORITHM: String = "secretHashAlgorithm"
    
    private var algorithm: String?
    private var publicKey: Foundation.Data?
    // The DirectActivationRequest encoding format
    private var format: String = "X.509" 
    private var secretHashAlgorithm: String?
    
    init() {}
    
    func getAlgorithm() -> String {
        return self.algorithm!
    }
    
    func setAlgorithm(algorithm: String) {
        self.algorithm = algorithm
    }
    
    func getPublicKey() -> Foundation.Data {
        return self.publicKey!
    }
    
    func setPublicKey(subjectPublicKey: Foundation.Data) {
        self.publicKey = subjectPublicKey;
    }
    
    func getFormat() -> String {
        return self.format
    }
    
    func setFormat(format: String) {
        self.format = format;
    }
    
    func getSecretHashAlgorithm() -> String {
        return self.secretHashAlgorithm!
    }
    
    func setSecretHashAlgorithm(secretHashAlgorithm: String) {
        self.secretHashAlgorithm = secretHashAlgorithm;
    }
    
    func toJsonString() throws -> String {
        let publicKeyEncodedString: String =
            self.publicKey!.base64EncodedString(
                options: NSData.Base64EncodingOptions(rawValue: 0))
        let resultStr = "{" +
            "\"\(SubjectPublicKeyInfo.FIELD_ALGORITHM)\":" +
            "\"\(self.algorithm!)\"," +
            "\"\(SubjectPublicKeyInfo.FIELD_PUBLIC_KEY)\":" +
            "\"\(publicKeyEncodedString)\"," +
            "\"\(SubjectPublicKeyInfo.FIELD_FORMAT)\":" +
            "\"\(self.format)\"," +
            "\"\(SubjectPublicKeyInfo.FIELD_SECRET_HASH_ALGORITHM)\":" +
            "\"\(self.secretHashAlgorithm!)\"" +
        "}"
        return resultStr
    }
    
    private func parseJsonStr(str: String) throws -> [String : AnyObject]? {
        if let data = str.data(using: .utf8) {
            return try JSONSerialization.jsonObject(with: data,
                                        options: []) as? [String : AnyObject]
        }
        return nil
    }
    
    func fromJson(data: Foundation.Data?) throws {
        if let dataIn = data {
            try fromJson(jsonDict: JSON(data: dataIn).dictionaryValue)
        }
    }
    
    func fromJson(jsonDict: [String: JSON]) throws {
        setFormat(format: jsonDict[SubjectPublicKeyInfo.FIELD_FORMAT]!.string!)
        setAlgorithm(algorithm: jsonDict[SubjectPublicKeyInfo.FIELD_ALGORITHM]!.string!);
        
        let encodedPublicKey: String =
            jsonDict[SubjectPublicKeyInfo.FIELD_PUBLIC_KEY]!.string!
        let publicKeyData: Foundation.Data = Foundation.Data(
                                        base64Encoded: encodedPublicKey,
                            options: NSData.Base64DecodingOptions(rawValue: 0))!
        setPublicKey(subjectPublicKey: publicKeyData)
        setSecretHashAlgorithm(secretHashAlgorithm:
            jsonDict[SubjectPublicKeyInfo.FIELD_SECRET_HASH_ALGORITHM]!.string!)
    }

    func toString() -> String {
        var result = "SubjectPublicKeyInfo{"
        result = result + "algorithm='" + self.algorithm! + "'"
        result = result + ", format='" + self.format + "'"
        result = result + ", secretHashAlgorithm='"
        result = result + self.secretHashAlgorithm! + "'" + "}"
        return result
    }
}

