/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

class NamedValueImpl: NamedValue {
    
    private var name: String
    private var value: AnyObject?
    private var nnext: NamedValue?
    
    init(name: String, value: AnyObject?) {
        self.name = name
        self.value = value
    }
    
    func getName() -> String {
        return name
    }
    
    func getValue() -> AnyObject? {
        return value
    }
    
    func next() -> NamedValue? {
        return nnext
    }
    
    func setNext(next: NamedValue) {
        self.nnext = next;
    }
    
}

class ChangeEventImpl: ChangeEvent {
    
    private var virtualDevice: VirtualDeviceImpl
    private var value: NamedValue
    
    init(virtualDevice: VirtualDeviceImpl, value: NamedValue) {
        self.virtualDevice = virtualDevice;
        self.value = value
    }
    
    func getVirtualDevice() -> VirtualDevice {
        return virtualDevice
    }
    
    func getNamedValue() -> NamedValue {
        return value
    }
}

class ErrorEventImpl: ErrorEvent {
    
    private var virtualDevice: VirtualDeviceImpl
    private var value: NamedValue
    private var message: String
    
    init(virtualDevice: VirtualDeviceImpl, value: NamedValue, message: String) {
        self.virtualDevice = virtualDevice
        self.value = value
        self.message = message
    }
    
    func getVirtualDevice() -> VirtualDevice {
        return virtualDevice
    }
    
    func getNamedValue() -> NamedValue {
        return value
    }
    
    func getMessage() -> String {
        return message
    }
}
